@@ -1,8 +1,9 @@
+#define IN_LIBEXSLT
 #include "libexslt/libexslt.h"
 
 #include <libxml/xmlversion.h>
 
-#if defined(WIN32) && !defined (__CYGWIN__)
+#if defined(WIN32) && !defined (__CYGWIN__) && (!__MINGW32__)
 #include <win32config.h>
 #else
 #include "config.h"
@@ -37,5 +38,6 @@ exsltRegisterAll (void) {
     exsltStrRegister();
     exsltDateRegister();
     exsltSaxonRegister();
+    exsltDynRegister();
 }
 
