@@ -15,6 +15,7 @@
  * daniel@veillard.com
  */
 
+#define IN_LIBXSLT
 #include "libxslt.h"
 
 #include <string.h>
@@ -25,6 +26,7 @@
 #include <libxml/valid.h>
 #include <libxml/hash.h>
 #include <libxml/uri.h>
+#include <libxml/encoding.h>
 #include <libxml/xmlerror.h>
 #include "xslt.h"
 #include "xsltutils.h"
@@ -65,10 +67,9 @@ xsltNewStylePreComp(xsltStylesheetPtr style, xsltStyleType type) {
 
     cur = (xsltStylePreCompPtr) xmlMalloc(sizeof(xsltStylePreComp));
     if (cur == NULL) {
-	xsltPrintErrorContext(NULL, style, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, NULL,
 		"xsltNewStylePreComp : malloc failed\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	return(NULL);
     }
     memset(cur, 0, sizeof(xsltStylePreComp));
@@ -120,10 +121,9 @@ xsltNewStylePreComp(xsltStylesheetPtr style, xsltStyleType type) {
 	    cur->func = NULL;break;
 	default:
 	if (cur->func == NULL) {
-	    xsltPrintErrorContext(NULL, style, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, NULL,
 		    "xsltNewStylePreComp : no function for type %d\n", type);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	}
     }
     cur->next = style->preComps;
@@ -193,16 +193,17 @@ xsltFreeStylePreComp(xsltStylePreCompPtr comp) {
  * xsltDocumentComp:
  * @style:  the XSLT stylesheet
  * @inst:  the instruction in the stylesheet
+ * @function:  unused
  *
  * Pre process an XSLT-1.1 document element
+ *
+ * Returns a precompiled data structure for the element
  */
 xsltElemPreCompPtr
 xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
 		 xsltTransformFunction function ATTRIBUTE_UNUSED) {
     xsltStylePreCompPtr comp;
     xmlChar *filename = NULL;
-    xmlChar *base = NULL;
-    xmlChar *URL = NULL;
 
     comp = xsltNewStylePreComp(style, XSLT_FUNC_DOCUMENT);
     if (comp == NULL)
@@ -248,32 +249,9 @@ xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
     if (!comp->has_filename) {
 	goto error;
     }
-
-    if (filename != NULL) {
-	/*
-	 * Compute output URL
-	 */
-	base = xmlNodeGetBase(inst->doc, inst);
-	URL = xmlBuildURI(filename, base);
-	if (URL == NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
-		"xsltDocumentComp: URL computation failed %s\n", filename);
-	    style->warnings++;
-	    comp->filename = xmlStrdup(filename);
-	} else {
-	    comp->filename = URL;
-	}
-    } else {
-	comp->filename = NULL;
-    }
+    comp->filename = filename;
 
 error:
-    if (base != NULL)
-	xmlFree(base);
-    if (filename != NULL)
-	xmlFree(filename);
-
     return ((xsltElemPreCompPtr) comp);
 }
 
@@ -294,7 +272,6 @@ static void
 xsltSortComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
 
-
     if ((style == NULL) || (inst == NULL))
 	return;
     comp = xsltNewStylePreComp(style, XSLT_FUNC_SORT);
@@ -312,11 +289,10 @@ xsltSortComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	else if (xmlStrEqual(comp->stype, (const xmlChar *) "number"))
 	    comp->number = 1;
 	else {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 		 "xsltSortComp: no support for data-type = %s\n", comp->stype);
-	    comp->number = -1;
-	    style->warnings++;
+	    comp->number = 0; /* use default */
+	    if (style != NULL) style->warnings++;
 	}
     }
     comp->order = xsltEvalStaticAttrValueTemplate(style, inst,
@@ -328,15 +304,31 @@ xsltSortComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	else if (xmlStrEqual(comp->order, (const xmlChar *) "descending"))
 	    comp->descending = 1;
 	else {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 		 "xsltSortComp: invalid value %s for order\n", comp->order);
-	    comp->descending = -1;
-	    style->warnings++;
+	    comp->descending = 0; /* use default */
+	    if (style != NULL) style->warnings++;
 	}
     }
-    /* TODO: xsl:sort lang attribute */
-    /* TODO: xsl:sort case-order attribute */
+    comp->case_order = xsltEvalStaticAttrValueTemplate(style, inst,
+			      (const xmlChar *)"case-order",
+			      XSLT_NAMESPACE, &comp->has_use);
+    if (comp->case_order != NULL) {
+	if (xmlStrEqual(comp->case_order, (const xmlChar *) "upper-first"))
+	    comp->lower_first = 0;
+	else if (xmlStrEqual(comp->case_order, (const xmlChar *) "lower-first"))
+	    comp->lower_first = 1;
+	else {
+	    xsltTransformError(NULL, style, inst,
+		 "xsltSortComp: invalid value %s for order\n", comp->order);
+	    comp->lower_first = 0; /* use default */
+	    if (style != NULL) style->warnings++;
+	}
+    }
+
+    comp->lang = xsltEvalStaticAttrValueTemplate(style, inst,
+				 (const xmlChar *)"lang",
+				 XSLT_NAMESPACE, &comp->has_lang);
 
     comp->select = xsltGetNsProp(inst,(const xmlChar *)"select", XSLT_NAMESPACE);
     if (comp->select == NULL) {
@@ -349,11 +341,15 @@ xsltSortComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     }
     comp->comp = xmlXPathCompile(comp->select);
     if (comp->comp == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsltSortComp: could not compile select expression '%s'\n",
 	                 comp->select);
-	style->errors++;
+	if (style != NULL) style->errors++;
+    }
+    if (inst->children != NULL) {
+	xsltTransformError(NULL, style, inst,
+	"xsl:sort : is not empty\n");
+	if (style != NULL) style->errors++;
     }
 }
 
@@ -415,10 +411,9 @@ xsltTextComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    comp->noescape = 1;
 	} else if (!xmlStrEqual(prop,
 				(const xmlChar *)"no")){
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 "xsl:text: disable-output-escaping allows only yes or no\n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
 	xmlFree(prop);
     }
@@ -452,7 +447,15 @@ xsltElementComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
 			 (const xmlChar *)"namespace",
 			 XSLT_NAMESPACE, &comp->has_ns);
+    if (comp->has_ns == 0) {
+	xmlNsPtr defaultNs;
 
+	defaultNs = xmlSearchNs(inst->doc, inst, NULL);
+	if (defaultNs != NULL) {
+	    comp->ns = xmlStrdup(defaultNs->href);
+	    comp->has_ns = 1;
+	}
+    }
     comp->use = xsltEvalStaticAttrValueTemplate(style, inst,
 		       (const xmlChar *)"use-attribute-sets",
 		       XSLT_NAMESPACE, &comp->has_use);
@@ -555,19 +558,17 @@ xsltCopyOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:copy-of : select is missing\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	return;
     }
     comp->comp = xmlXPathCompile(comp->select);
     if (comp->comp == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:copy-of : could not compile select expression '%s'\n",
 	                 comp->select);
-	style->errors++;
+	if (style != NULL) style->errors++;
     }
 }
 
@@ -599,29 +600,26 @@ xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    comp->noescape = 1;
 	} else if (!xmlStrEqual(prop,
 				(const xmlChar *)"no")){
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 "xsl:value-of : disable-output-escaping allows only yes or no\n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
 	xmlFree(prop);
     }
     comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:value-of : select is missing\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	return;
     }
     comp->comp = xmlXPathCompile(comp->select);
     if (comp->comp == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:value-of : could not compile select expression '%s'\n",
 	                 comp->select);
-	style->errors++;
+	if (style != NULL) style->errors++;
     }
 }
 
@@ -650,16 +648,15 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
      */
     prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:with-param : name is missing\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
     } else {
         const xmlChar *URI;
 
 	URI = xsltGetQNameURI(inst, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	} else {
 	    comp->name = prop;
 	    comp->has_name = 1;
@@ -677,17 +674,15 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     if (comp->select != NULL) {
 	comp->comp = xmlXPathCompile(comp->select);
 	if (comp->comp == NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 		 "xsl:param : could not compile select expression '%s'\n",
 			     comp->select);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	}
 	if (inst->children != NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 	    "xsl:param : content should be empty since select is present \n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
     }
 }
@@ -740,10 +735,9 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    xmlStrEqual(prop, BAD_CAST("any"))) {
 	    comp->numdata.level = prop;
 	} else {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 			 "xsl:number : invalid value %s for level\n", prop);
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	    xmlFree(prop);
 	}
     }
@@ -757,22 +751,19 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
     prop = xsltGetNsProp(cur, (const xmlChar *)"letter-value", XSLT_NAMESPACE);
     if (prop != NULL) {
 	if (xmlStrEqual(prop, BAD_CAST("alphabetic"))) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 		 "xsl:number : letter-value 'alphabetic' not implemented\n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	    XSLT_TODO; /* xsl:number letter-value attribute alphabetic */
 	} else if (xmlStrEqual(prop, BAD_CAST("traditional"))) {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 		 "xsl:number : letter-value 'traditional' not implemented\n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	    XSLT_TODO; /* xsl:number letter-value attribute traditional */
 	} else {
-	    xsltPrintErrorContext(NULL, style, cur);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, cur,
 		     "xsl:number : invalid value %s for letter-value\n", prop);
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
 	xmlFree(prop);
     }
@@ -780,7 +771,9 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
     prop = xsltGetNsProp(cur, (const xmlChar *)"grouping-separator",
 	                XSLT_NAMESPACE);
     if (prop != NULL) {
-	comp->numdata.groupingCharacter = prop[0];
+        comp->numdata.groupingCharacterLen = xmlStrlen(prop);
+	comp->numdata.groupingCharacter =
+	    xsltGetUTF8Char(prop, &(comp->numdata.groupingCharacterLen));
 	xmlFree(prop);
     }
     
@@ -846,16 +839,15 @@ xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
      */
     prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:call-template : name is missing\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
     } else {
         const xmlChar *URI;
 
 	URI = xsltGetQNameURI(inst, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	} else {
 	    comp->name = prop;
 	    comp->has_name = 1;
@@ -899,7 +891,7 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 	URI = xsltGetQNameURI(inst, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	} else {
 	    comp->mode = prop;
 	    if (URI != NULL) {
@@ -914,11 +906,10 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     if (comp->select != NULL) {
 	comp->comp = xmlXPathCompile(comp->select);
 	if (comp->comp == NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
      "xsl:apply-templates : could not compile select expression '%s'\n",
 			     comp->select);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	}
     }
 
@@ -966,19 +957,17 @@ xsltIfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
     comp->test = xsltGetNsProp(inst, (const xmlChar *)"test", XSLT_NAMESPACE);
     if (comp->test == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:if : test is not defined\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	return;
     }
     comp->comp = xmlXPathCompile(comp->test);
     if (comp->comp == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:if : could not compile test expression '%s'\n",
 	                 comp->test);
-	style->errors++;
+	if (style != NULL) style->errors++;
     }
 }
 
@@ -1003,19 +992,17 @@ xsltWhenComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
     comp->test = xsltGetNsProp(inst, (const xmlChar *)"test", XSLT_NAMESPACE);
     if (comp->test == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:when : test is not defined\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
 	return;
     }
     comp->comp = xmlXPathCompile(comp->test);
     if (comp->comp == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:when : could not compile test expression '%s'\n",
 	                 comp->test);
-	style->errors++;
+	if (style != NULL) style->errors++;
     }
 }
 
@@ -1041,18 +1028,16 @@ xsltForEachComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 		"xsl:for-each : select is missing\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
     } else {
 	comp->comp = xmlXPathCompile(comp->select);
 	if (comp->comp == NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
      "xsl:for-each : could not compile select expression '%s'\n",
 			     comp->select);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	}
     }
     /* TODO: handle and skip the xsl:sort */
@@ -1083,16 +1068,15 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
      */
     prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:variable : name is missing\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
     } else {
         const xmlChar *URI;
 
 	URI = xsltGetQNameURI(inst, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	} else {
 	    comp->name = prop;
 	    comp->has_name = 1;
@@ -1110,17 +1094,15 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     if (comp->select != NULL) {
 	comp->comp = xmlXPathCompile(comp->select);
 	if (comp->comp == NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 		 "xsl:variable : could not compile select expression '%s'\n",
 			     comp->select);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	}
 	if (inst->children != NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 	"xsl:variable : content should be empty since select is present \n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
     }
 }
@@ -1150,16 +1132,15 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
      */
     prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
-	xsltPrintErrorContext(NULL, style, inst);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(NULL, style, inst,
 	     "xsl:param : name is missing\n");
-	style->errors++;
+	if (style != NULL) style->errors++;
     } else {
         const xmlChar *URI;
 
 	URI = xsltGetQNameURI(inst, &prop);
 	if (prop == NULL) {
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	} else {
 	    comp->name = prop;
 	    comp->has_name = 1;
@@ -1177,17 +1158,15 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     if (comp->select != NULL) {
 	comp->comp = xmlXPathCompile(comp->select);
 	if (comp->comp == NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 		 "xsl:param : could not compile select expression '%s'\n",
 			     comp->select);
-	    style->errors++;
+	    if (style != NULL) style->errors++;
 	}
 	if (inst->children != NULL) {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 	"xsl:param : content should be empty since select is present \n");
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
     }
 }
@@ -1325,10 +1304,9 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    inst->_private = (void *) xsltDocumentComp(style, inst,
 				(xsltTransformFunction) xsltDocumentElem);
 	} else {
-	    xsltPrintErrorContext(NULL, style, inst);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(NULL, style, inst,
 		 "xsltStylePreCompute: unknown xsl:%s\n", inst->name);
-	    style->warnings++;
+	    if (style != NULL) style->warnings++;
 	}
 	/*
 	 * Add the namespace lookup here, this code can be shared by
