@@ -185,6 +185,7 @@ skipped_modules = {
     'list': None,
     'threads': None,
     'xpointer': None,
+    'transform': None,
 }
 skipped_types = {
     'int *': "usually a return type",
@@ -267,6 +268,8 @@ py_types = {
     'FILE *': ('O', "File", "FILEPtr", "FILE *", "libxml_"),
     'xsltTransformContextPtr':  ('O', "transformCtxt", "xsltTransformContextPtr", "xsltTransformContextPtr", "libxslt_"),
     'xsltTransformContext *':  ('O', "transformCtxt", "xsltTransformContextPtr", "xsltTransformContextPtr", "libxslt_"),
+    'xsltStylePreCompPtr':  ('O', "compiledStyle", "xsltStylePreCompPtr", "xsltStylePreCompPtr", "libxslt_"),
+    'xsltStylePreComp *':  ('O', "compiledStyle", "xsltStylePreCompPtr", "xsltStylePreCompPtr", "libxslt_"),
     'xsltStylesheetPtr':  ('O', "stylesheet", "xsltStylesheetPtr", "xsltStylesheetPtr", "libxslt_"),
     'xsltStylesheet *':  ('O', "stylesheet", "xsltStylesheetPtr", "xsltStylesheetPtr", "libxslt_"),
     'xmlXPathContext *':  ('O', "xpathContext", "xmlXPathContextPtr", "xmlXPathContextPtr", "libxslt_"),
@@ -403,10 +406,11 @@ def print_function_wrapper(name, output, export, include):
         return 1
 
     output.write("PyObject *\n")
-    output.write("libxslt_%s(ATTRIBUTE_UNUSED PyObject *self," % (name))
+    output.write("libxslt_%s(PyObject *self ATTRIBUTE_UNUSED," % (name))
+    output.write(" PyObject *args")
     if format == "":
-	output.write("ATTRIBUTE_UNUSED ")
-    output.write(" PyObject *args) {\n")
+	output.write(" ATTRIBUTE_UNUSED")
+    output.write(") {\n")
     if ret[0] != 'void':
         output.write("    PyObject *py_retval;\n")
     if c_return != "":
